;DUMBO
;Phase Vocoder used to combine birds & elephants
;Saint-Saens Carnival of the Animals
;LEE ZAKIAN	9/1/2003 1:27:06 PM

sr 		= 44100
kr 		= 4410
ksmps 		= 10
nchnls 		= 2
zakinit   	10, 10

/*
Phase vocoder instruments, to cross-synthesize 2, 3, or 4 analysed samples. 
Each instrument has a .pvc file(s) sent through the opcodes PVBUFREAD and PVCROSS.

===CONFIGURATIONS
1.) 2 samples:	One analyzed sample in each instrument
2.) 3 samples:	One analyzed samples in Instr 1, two analyzed samples in Instr 2
3.) 4 samples:	Two analyzed samples in each instrument

===AMPLITUDE
"kamp" controls the dynamic level
Set to 0 to mute
Set to 1 for no change
Use "LINE" to create a gradual crescendo or decrescendo
Use "LINSEG" for varied amplitude

===PITCH
"ifmod" is used to tune instrument 1 to instrument 2
Set to 1 for no change
Set from >1 to 2 to raise the pitch up to 1 octave
Set from <1 to .5 to lower the pitch up to 1 octave
Use "CPSPCH" to designate transposition by steps

"kfmod" is used for variable pitch changes
Use "LINE" for gradual pitch change
Use "LINSEG" for varied pitch

===TIME
"ktime" is used to change  the tempo of the analyzed samples
Using "LINE", a value equal to the sample length will play it unchanged
Using "LINE", a value > the sample length makes it play faster
Using "LINE", a value < than the sample length makes it play slower

Using the ZAK system, .pvc files go through EQ and REVERB instruments,
and out a mixer. Several panning options  are explained in the mixer instrument.
The EQ instrument is especially helpful if the phase vocoder process has altered the
audio spectrum in an unsatisfactory way. The EQ and REVERB instruments may be
bypassed if the iout arguments in the phase vocoder instruments are set to the input
channels of the mixer.
*/
  
	instr     1
	
iout		=	p4
kamp1  	line	1, p3, 0 
kamp2  	linseg	0, p3/3, .5, p3/3, 1, p3/3, 0		
ifmod     	= 	cpspch(8.00)/cpspch(7.11) 
 
ktime1    	line	0, 	18.51, 	10	
ktime2    	line	0,	18.51,	18.51

          		pvbufread 	ktime1, "SSbird.pvc"
apvc      	pvcross   	ktime2, ifmod, "SSbird.pvc", kamp1, kamp2
		zawm       	apvc, iout
          	endin

	instr     2	

iout		=	p4
kamp1  	linseg	.3, p3/5, .5, p3/5, .7, p3/5, .5, p3/5, .3, p3/5, 0 
kamp2  	linseg	.3, p3/5, .5, p3/5, .7, p3/5, .5, p3/5, .3, p3/5, 0	
ifmod     	= 	cpspch(8.00)/cpspch(8.01)

ktime1    	line	0, 	25.54, 	25.54	
ktime2    	line	0,	25.54,	25.54

          		pvbufread 	ktime1, "SSelephant2.pvc"
apvc      	pvcross   	ktime2, ifmod, "SSelephant2.pvc", kamp1, kamp2
		zawm       	apvc, iout
          	endin

	instr 88		;hp01	Variable HP filter, set with p8

ivol		=	ampdb(p4)
ilevel		=	p5
iinhp01	=	p6
iouthp01	=	p7	;to mixer or more processing
ihighpass	=	p8

kdclk		linseg  	0, .2, 1, p3-.4, 1, .2, 0	;declick envelope
asig		zar	iinhp01

af1		butterhp	asig, ihighpass	
af2		butterhp	af1, ihighpass
af3		butterhp	af2, ihighpass
af4		butterhp	af3, ihighpass
af5		=		af4*p5

aout		=		(af4+af5)*ivol	
		zawm		aout*kdclk, iouthp01
		endin


	instr 89		;lp01	Variable LP filter, set with p8

ivol		=	ampdb(p4)
ilevel		=	p5
iinlp01		=	p6
ioutlp01	=	p7	;to mixer or more processing
ilowpass	=	p8

kdclk		linseg  	0, .2, 1, p3-.4, 1, .2, 0	;declick envelope
asig		zar		iinlp01

af1		butterlp	asig, ilowpass	
af2		butterlp	af1, ilowpass
af3		butterlp	af2, ilowpass
af4		butterlp	af3, ilowpass
af5		=	af4*p5

aout		=	(af4+af5)*ivol	
		zawm	aout*kdclk, ioutlp01
		endin

	instr 98		;REVERB	Instr 1 Left, Instr 2 Right

iinL		=	p4
iinR		=	p5
ioutL		=	p6
ioutR		=	p7	
irvbtimeL	=	p8
irvbtimeR	=	p9	
iwetL		=	p10
idryL		=	1-p10
iwetR		=	p11
idryR		=	1-p11
ihfrollL	=	p12	;low pass filter: high frequency rolloff
ihfrollR	=	p13
ihfdiffL	=	p14	;high frequency diffusion
ihfdiffR	=	p15
asigL		zar	iinL
asigR		zar	iinR
aoutrevL	nreverb	asigL*iwetL, p8, p14
aoutrevR	nreverb	asigR*iwetR, p9, p15
aoutL		tone	aoutrevL, p12
aoutR		tone	aoutrevR, p13
		zawm	(aoutL+(asigL*idryL)),ioutL
		zawm	(aoutR+(asigR*idryR)), ioutR	
	endin

	instr 100	;MIXER

/*
	3 PANNING OPTIONS

===Single Location	1 to 0
kpanL		=	1 			pan hard left 
kpanlR		=	0 			pan hard right

===Movement In One Direction
kpanL		line	0, p3, 1 	 	pan right to left
kpanR		line	1, p3, 0			pan left to right

===Movement In Multiple Directions
kpanL		linseg	1, p3/2, 0, p3/2, 1	pan left to right to left
kpanR		linseg	0, p3/2, 1, p3/2, 0	pan right to left to right
*/
 
;++CHANNEL 1	Bird
a1		zar	p4	;IN
igL		init	p5	;GAIN
kpanL		linseg	0, p3/5, 1, p3/5, 0, p3/5, 1, p3/5, 0, p3/5, 1	;PAN
						
;=====
;++CHANNEL 2	Elephant
a2		zar	p6	;IN
igR		init	p7	;GAIN
kpanR		linseg	1, p3/2, 0, p3/2, 1	;PAN

aoutL		=	((igL*a1)*(kpanL))+((igL*a2)*(1-kpanL))
aoutR		=	((igR*a1)*(kpanR))+((igR*a2)*(1-kpanR))
		outs	aoutL, aoutR
		zacl	0, 10
		endin

